/****************************************************************************
*
* (C) 2000-2003 by BECK IPC GmbH and PWR Solutions GbR
*
*  BECK IPC GmbH                    PWR Solutions GbR
*  Garbenheimerstr. 36-38           Euckenstrasse 17
*  D-35578 Wetzlar                  D-81369 Mnchen
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module        : I2C_API.C
* Function      : I2C API Library
* ---------------------------------------------------------------------------
* History       :
*  Vx.yy                   Author  Changes
*  V1.00      17.07.00     roth    Create
* ---------------------------------------------------------------------------
*  Published on the BECK WebSite with kind permission of Bernhard Roth.

$Header: I2C_API.C, 9, 12.03.2004 16:38:59, Andre Pribil$

****************************************************************************/



/****************************************************************************/
//includes
/****************************************************************************/

#include <dos.h>

#include <common.h>
#include <I2C_API.H>
#include "I2C_priv.H"

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 I2C API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces I2C_API\.H><efont> | 
    <tab><linkto>#Control<linkmid>Control<linktail><tab>
    <linkto>#Transmit<linkmid>Transmit<linktail><tab>
    <linkto>#Receive<linkmid>Receive<linktail><tab>
 @libcat1  Control   | i2c80 | I2C_init | Initialize the I2C Bus |
 @libcat1  Control   | i2c81 | I2C_scan | Scan for I2C slave devices |
 @libcat1  Control   | i2c84 | I2C_release | Release I2C bus |
 @libcat1  Control   | i2c8b | I2C_restart | Restart I2C |
 @libcat1  Control   | i2c8e | I2C_select_clock_pin | Select I2C Clock Pin |
 @libcat1  Control   | i2c8f | I2C_select_data_pin  | Select I2C Data Pin |

 @libcat2  Transmit  | i2c82t | I2C_transmit_char  | Send a single character |
 @libcat2  Transmit  | i2c83t | I2C_transmit_block | Send a block of characters |

 @libcat3  Receive   | i2c82r | I2C_receive_char  | Receive a single character |
 @libcat3  Receive   | i2c83r | I2C_receive_block | Receive block of characters |

*/


/*************************************************************************/
//Init the I2C Interface
/*************************************************************************/
#ifdef _i2c80_
/*
 @doc DOC_i2c80_
 ---------------------------------------------------------------
 I2C init I2C-function
 ---------------------------------------------------------------
 @clib  I2C_init | Initialize the I2C Bus | 
        <cfont>void I2C_init ( void );<efont> |
 @parm  byte |  | -- none --
 @rdesc -- none --
 @comm  This function configures two of the
        programmable I/O (PIO) pins for usage as I2C bus data and clock signals.<nl><nl>
        The user can specify which two PIO are used for I2C
        <linkto>i2c8e.htm<linkmid>clock<linktail> and <linkto>i2c8f.htm<linkmid>data<linktail>.<s2>
        After calling this initialization function, these two pins will no longer be available
        as PIO pins unless the <linkto>../../hardware.htm#0xA20x82<linkmid>PFE Enable<linktail> function
        is called for these pins following this function call.
 @Also  i2c8e.htm | I2C_select_clock_pin |
        i2c8f.htm | I2C_select_data_pin  | |
 @Related PFE:  | ../../hardware.htm#0xA20x82 | Enable | Programmable I/O Pins
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x80<linkmid>documentation<linktail>
        for more details.
*/
void I2C_init (void)
{
    _AH = I2C_INIT ;
    asm { int I2CINT  } ;
}
#endif

/*************************************************************************/
//Scan for interfaces on I2C
/*************************************************************************/
#ifdef _i2c81_
/*
 @doc DOC_i2c81_
 ---------------------------------------------------------------
 I2C scan I2C-devices
 ---------------------------------------------------------------
 @clib  I2C_scan | Scan for I2C slave devices |
        <cfont>unsigned char I2C_scan ( unsigned char start_addr,<nl>
        <tab><tab><tab><tab><tab><tab> unsigned char end_addr ); <efont> |
 @parm  byte | start_addr  | First slave address (even address, LSB=0)
 @parm  byte | end_addr  | Last slave address (even address, LSB=0)
 @rdesc 0 => no slave found<nl>
        Non-zero => address of the first slaved device found <nl>
 @comm  This is an iterator function which is called repetitively to determine
        all connected slaves.<s2> Specify on each successive call a new restricted
        slave address range until no further address is returned by this function.<s2>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x81<linkmid>documentation<linktail>
        for more details.
*/
unsigned char I2C_scan (unsigned char start_addr, unsigned char end_addr)
{
    unsigned char result ;

    _CL = end_addr ;
    _AL = start_addr ;
    _AH = I2C_SCAN ;
    asm { int I2CINT  } ;
    return _AL ;
}
#endif

/*************************************************************************/
//Receive one Character
/*************************************************************************/
#ifdef _i2c82r_
/*
@doc DOC_i2c82r_
 ---------------------------------------------------------------
  I2C receive character
 ---------------------------------------------------------------
 @clib  I2C_receive_char | Receive a single character |
        <cfont>int I2C_receive_char ( unsigned char slave,<nl>
        <tab><tab><tab><tab><tab>&nbsp;&nbsp; char far *char_in,<nl>
        <tab><tab><tab><tab><tab>&nbsp;&nbsp; unsigned char lastchar );<efont> |
 @parm  byte | slave  | 7 bit slave address in B7-B1, B0=Don't care
 @parm  byte | char_in  | Pointer to one character buffer provided by caller where
        received character will be placed.
 @parm  byte | not_last  | = 0 for last char to be received, otherwise non-zero. <nl>
 @rdesc 0: Success, received char is output to 'char_in' location.<nl>
        5: Bus allocated<nl>
        6: Bus arbitration failed<nl>
        7: Bus error<nl>
        8: Timeout<nl>
        9: Slave faulty or not available<ule>
 @comm  The IPC@CHIP is the I2C bus master.<nl><nl>
 @Also  i2c83r.htm |  I2C_receive_block |
        i2c83t.htm |  I2C_transmit_char | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x82<linkmid>documentation<linktail>
        for more details.
*/
int I2C_receive_char (unsigned char slave,
                      char *char_in,
                      unsigned char not_last)
{
    unsigned char  flags;
    unsigned char result ;
    char the_char ;

    _CL = not_last ;         // = 0 if last char.
    _AX = (I2C_TRANS_RECV_CHAR << 8) | 0x01 | (unsigned)slave  ;
    asm {
      int I2CINT
      lahf
    } ;
    flags    = _AH ;    // Before they gets lost.
    result   = _AL ;
    the_char = _CH ;

    if (flags & CARRY_FLAG)
    {
        the_char = 0 ;
    }
    else
    {
        result = 0 ;
    }

    *char_in = the_char ;               // Set output parameter
    return (int) ((unsigned int) result) ;
}
#endif

/*************************************************************************/
//Transmit one character
/*************************************************************************/
#ifdef _i2c82t_
/*
@doc DOC_i2c82t_
 ---------------------------------------------------------------
  I2C send character
 ---------------------------------------------------------------
 @clib  I2C_transmit_char | Send a single character |
        <cfont>int I2C_transmit_char ( unsigned char slave,<nl>
        <tab><tab><tab><tab><tab><tab>char char_out );<efont> |
 @parm  byte | slave  | 7 bit slave address in B7-B1, B0=Don't care
 @parm  byte | char_out  | Byte to transmit.
 @rdesc 0: Success<nl>
        5: Bus allocated<nl>
        6: Bus arbitration failed<nl>
        7: Bus error<nl>
        8: Timeout<nl>
        9: Slave faulty or not available<ule>
 @comm  The IPC@CHIP is the I2C bus master.<nl><nl>
 @Also  i2c83t.htm |  I2C_send_block   |
        i2c82r.htm |  I2C_receive_char | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x82<linkmid>documentation<linktail>
        for more details.
*/
int I2C_transmit_char (unsigned char slave, char char_out)
{
    unsigned char  flags;
    int result ;

    _CL = char_out ;

    _AX = (I2C_TRANS_RECV_CHAR << 8)
        | (unsigned)(slave & 0xFE) ;
    asm {
        int I2CINT
        lahf
    } ;
    flags    = _AH ;    // Before they gets lost.
    result   = (int)((unsigned int) _AL) ;

    if ((flags & CARRY_FLAG) == 0)
    {
        result = 0 ;
    }
    return result ;
}
#endif

/*************************************************************************/
//Receive Block
/*************************************************************************/
#ifdef _i2c83r_
/*
 @doc DOC_i2c83r_
 ---------------------------------------------------------------
 I2C receive block
 ---------------------------------------------------------------
 @clib  I2C_receive_block | Receive block of data on I2C bus |
        <cfont>int I2C_receive_block ( unsigned char slave,<nl>
        <tab><tab><tab><tab><tab><tab>char far *buffer,<nl>
        <tab><tab><tab><tab><tab><tab>int length );<efont> |
 @parm  byte   | slave  | 7 bit slave address in B7..B1; B0 n/a
 @parm  void * | buffer | Pointer to where data will be placed
 @parm  int    | length  | Number of bytes to receive
 @rdesc result  -  0 = Success<nl>
        <tab>non-zero indicates error:<nl><ul>
        5: Bus allocated<nl>
        6: Bus arbitration failed<nl>
        7: Bus error<nl>
        8: Timeout<nl>
        9: Slave faulty or not available<nl>
        <ule>
 @comm  This function will dwell until either 'length' bytes are received and stored in the buffer, or
        until an error occurs.<s2>
 @Also  i2c82r.htm |  I2C_receive_char |
        i2c83t.htm |  I2C_transmit_block | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x83<linkmid>documentation<linktail>
        for more details.
*/
int I2C_receive_block (unsigned char slave, char far * buffer, int length)
{
    unsigned char  flags;
    int result ;
    asm { LES  BX,buffer }
    _CX = length ;
    _AX = (I2C_TRANS_RECV_BLOCK << 8) | 0x01
        | (unsigned)slave  ;
    asm {
        int I2CINT
        lahf
    } ;
    flags    = _AH ;    // Before they gets lost.
    result   = (int)((unsigned int) _AL) ;

    if ((flags & CARRY_FLAG) == 0)
    {
        result = 0 ;
    }
    return result ;
}
#endif

/*************************************************************************/
//Transmit Block
/*************************************************************************/
#ifdef _i2c83t_
/*
 @doc DOC_i2c83t_
 ---------------------------------------------------------------
 I2C transmit block
 ---------------------------------------------------------------
 @clib  I2C_transmit_block | Send block of data on I2C bus |
        <cfont>int I2C_transmit_block ( unsigned char slave,<nl>
        <tab><tab><tab><tab><tab><tab> const char far *buffer,<nl>
        <tab><tab><tab><tab><tab><tab> int length );<efont> |
 @parm  byte   | slave  | 7 bit slave address in B7..B1; B0 n/a
 @parm  void * | buffer | Pointer to data source
 @parm  int    | length  | Number of bytes to send from buffer
 @rdesc result  -  0 = Success<nl>
        <tab>non-zero indicates error:<nl><ul>
        5: Bus allocated<nl>
        6: Bus arbitration failed<nl>
        7: Bus error<nl>
        8: Timeout<nl>
        9: Slave faulty or not available<nl>
        <ule>
 @comm This function will dwell until 'length' bytes from the buffer are transmitted or until an
       error occurs.
 @Also i2c82t.htm |  I2C_transmit_char |
       i2c83r.htm |  I2C_receive_block | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x83<linkmid>documentation<linktail>
        for more details.
*/
int I2C_transmit_block (unsigned char slave, const char far * buffer, int length)
{
    unsigned char  flags;
    int result ;
    asm { LES  BX,buffer }
    _CX = length ;
    _AX = (I2C_TRANS_RECV_BLOCK << 8)
        | (unsigned)(slave & 0xFE) ;
    asm {
        int I2CINT
        lahf
    } ;
    flags    = _AH ;    // Before they gets lost.
    result   = (int)((unsigned int) _AL) ;

    if ((flags & CARRY_FLAG) == 0)
    {
        result = 0 ;
    }
    return result ;
}
#endif

/*************************************************************************/
//Release I2C bus
/*************************************************************************/
#ifdef _i2c84_
/*
 @doc DOC_i2c84_
 ---------------------------------------------------------------
 Release I2C bus
 ---------------------------------------------------------------
 @clib  I2C_release | Release I2C Bus |
        <cfont>void I2C_release ( void ); <efont>  |
 @parm  byte |  | -- none --
 @rdesc -- none --
 @comm  This function provides the I2C bus stop condition:
        a data low to high transition during clock high.<s2> Both clock and
        data output pins are left tri-stated.
 @Also  i2c8b.htm |  I2C_restart | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x84<linkmid>documentation<linktail>
        for more details.
*/
void I2C_release (void)
{
    _AX = I2C_RELEASE << 8 ;        // AL = 0
    asm { int I2CINT  } ;
}
#endif

/*************************************************************************/
//Restart I2C
/*************************************************************************/
#ifdef _i2c8b_
/*
 @doc DOC_i2c8b_
 ---------------------------------------------------------------
 Restart I2C
 ---------------------------------------------------------------
 @clib  I2C_restart | Restart I2C Bus |
        <cfont>void I2C_restart ( void );<efont>  |
 @parm  byte | | -- none --
 @rdesc -- none --
 @comm  This function provides the I2C bus start condition:
        a data high to low transition during clock high.
 @Also  i2c84.htm |  I2C_release | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x8B<linkmid>documentation<linktail>
        for more details.
*/
void I2C_restart (void)
{
    _AX = I2C_RESTART << 8 ;         // AL = 0
    asm { int I2CINT  } ;
}
#endif

/*************************************************************************/
// Select I2C Clock Pin
/*************************************************************************/
#ifdef _i2c8e_
/*
 @doc DOC_i2c8e_
 ---------------------------------------------------------------
 Select I2C Clock Pin
 ---------------------------------------------------------------
 @clib  I2C_select_clock_pin | Select I2C Clock Pin |
        <cfont>void I2C_select_clock_pin ( unsigned char pio_no );<efont> |
 @parm  byte   | pio_no  | PIO pin number, [0..13]
 @rdesc -- none --
 @comm  The default I2C clock pin is PIO 0.<nl><nl>
        To change the I2CCLK pin this function must be called before the
        <linkto>i2c80.htm<linkmid>I2C_init<linktail> function is called.
 @Also  i2c8f.htm |  I2C_select_data_pin |
        i2c80.htm |  I2C_init |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x8E<linkmid>documentation<linktail>
        for more details.
*/
void I2C_select_clock_pin(unsigned char pio_no)
{
    _AL = pio_no ;
    _AH = I2C_SELECT_CLK_PIN ;
    asm { int I2CINT  } ;
}
#endif

/*************************************************************************/
//Select I2C data Pin
/*************************************************************************/
#ifdef _i2c8f_
/*
 @doc DOC_i2c8f_
 ---------------------------------------------------------------
 Select I2C Data Pin
 ---------------------------------------------------------------
 @clib  I2C_select_data_pin | Select I2C Data Pin |
        <cfont>void I2C_select_data_pin ( unsigned char pio_no );<efont> |
 @parm  byte   | pio_no  | PIO pin number, [0..13]
 @rdesc -- none --
 @comm  The default I2C data pin is PIO 1.<nl><nl>
        To change the I2CDAT pin this function must be called before the
        <linkto>i2c80.htm<linkmid>I2C_init<linktail> function is called.
 @Also  i2c8e.htm |  I2C_select_clock_pin |
        i2c80.htm |  I2C_init |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../i2c_spi.htm#0xAA0x8F<linkmid>documentation<linktail>
        for more details.
*/
void I2C_select_data_pin(unsigned char pio_no)
{
    _AL = pio_no ;
    _AH = I2C_SELECT_DATA_PIN ;
    asm { int I2CINT  } ;
}
#endif

// End I2C_API.C ////////////////////////

