/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     event.c
* Function:   This program demonstrates the usage of the RTOS-API of the IPC@Chip.
*             Two tasks will be created that, when receiving an event, will print
*             a message to STDOUT. When the user presses Return the main task will
*             trigger the event. So every time the user presses Return both tasks
*             will print its message to STDOUT.
*             The program can be exited by pressing Esc.
*             Printing the messages is protected by a resource semaphore.
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\rtos\Event\EVENT.C, 5, 31.03.2004 11:15:39, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include "clib.h"



/******************************************************************************
* Constants
******************************************************************************/
#define TASKSTACKSIZE  2048  // Size of stack for created tasks in bytes



/******************************************************************************
* Prototypes
******************************************************************************/
void huge task0(void );
void huge task1(void );



/******************************************************************************
* Global variables
******************************************************************************/
// Needed to control the tasks
int task0Id,                     // ID of task 0
    task1Id;                     // ID of task 1
unsigned int task0Stack[TASKSTACKSIZE/sizeof(unsigned int)];
unsigned int task1Stack[TASKSTACKSIZE/sizeof(unsigned int)];
                                 // Stack for the resp. task
unsigned char task0Running = 0,  // Task 0 will run while this is != 0
              task1Running = 0;  // Task 1 will run while this is != 0
TaskDefBlock task0DefBlock = {
                               task0,              // Function to be executed in
                                                   // the task
                               {'T','A','S','0'},  // Name of the task
                               &task0Stack[TASKSTACKSIZE/sizeof(unsigned int)],
                                                   // Top of stack
                               TASKSTACKSIZE,      // Size of the stack
                               0,                  // Not supported
                               26,                 // Normal priority
                               0,                  // Time slice (not needed here)
                               0, 0, 0, 0          // Mailbox depth (not needed here)
                             },  // Information on task 0
             task1DefBlock = {
                               task1,              // Function to be executed in
                                                   // the task
                               {'T','A','S','1'},  // Name of the task
                               &task1Stack[TASKSTACKSIZE/sizeof(unsigned int)],
                                                   // Top of stack
                               TASKSTACKSIZE,      // Size of the stack
                               0,                  // Not supported
                               26,                 // Normal priority
                               0,                  // Time slice (not needed here)
                               0, 0, 0, 0          // Mailbox depth (not needed here)
                             };  // Information on task 1

// Needed to control the events
unsigned int eventgroupId;       // ID of the event group
char eventgroupName[] = { 'E', 'V', 'E', '0' };
                                 // Name of the event group

// Needed to control the semaphore that protects printing to STDOUT
int semaphoreId;                 // ID of the semaphore
char  semaphoreName[] = {'S','E','M','0'};
                                 // Name of the semaphore



/******************************************************************************
* task0()
*
* This function will be executed in a seperate task.
* It prints out a message each time an event occures.
******************************************************************************/
void huge task0(void )
{
  static unsigned int nEvents = 0;  // Number of events since program start
  char message[100];                // Message to be printed out
  long timeout;


  RTX_Wait_Event waitEvent = {
                               0x0001,  // Event mask, only event flag 0 is relevant
                               0xFFFF,  // Clock event when flag has value 1
                               0,       // 0: Only the flags must match
                               10000L   // Timeout
                             };  // Information on the event


  // task0Running will be set to 0 on program exit by the main task
  task0Running = 1;
  while( task0Running )
  {
    // Wait for event
    switch( RTX_Wait_For_Event( eventgroupId, &waitEvent ) )
    {
      case 0:
                sprintf( message, "Event has been clocked for the %u. time.", ++nEvents );
                break;
      case -27:
                strcpy( message, "Timeout while waiting for event" );
                break;
      default:
                strcpy( message, "Timeout while waiting for event" );

    }

    // Print message
    timeout=0L;
    if( RTX_Reserve_Sem( semaphoreId, &timeout ) == 0 )  // Wait for semaphore endlessly
    {
      printf( "\r\nTask %i: %s", RTX_Get_TaskID(), message );
      RTX_Release_Sem( semaphoreId );  // Release semaphore
    }
  }  // while( task0Running )

  // The task deletes itself after task0Running has been set to 0
  if( RTX_Delete_Task( RTX_Get_TaskID() ) != 0 )
  // Of course this is superfluous for this code will not be reached if the task
  // has been deleted...
  {
    printf( "\r\nError: Task %i could not be deleted.", RTX_Get_TaskID() );
  }
}



/******************************************************************************
* task1()
*
* This function will be executed in a seperate task.
* It prints out a message each time an event occures.
******************************************************************************/
void huge task1(void )
{
  static unsigned int nEvents = 0;  // Number of events since program start
  long timeout;
  char message[100];                // Message to be printed out
  RTX_Wait_Event waitEvent = {
                               0x0001,  // Event mask, only event flag 0 is relevant
                               0xFFFF,  // Clock event when flag has value 1
                               0,       // 0: Only the flags must match
                               10000L   // Timeout
                             };  // Information on the event


  // task1Running will be set to 0 on program exit by the main task
  task1Running = 1;
  while( task1Running )
  {
    // Wait for event
    // Wait for event
    switch( RTX_Wait_For_Event( eventgroupId, &waitEvent ) )
    {
      case 0:
                sprintf( message, "Event has been clocked for the %u. time.", ++nEvents );
                break;
      case -27:
                strcpy( message, "Timeout while waiting for event" );
                break;
      default:
                strcpy( message, "Timeout while waiting for event" );

    }

    // Print message
    timeout=0L;
    if( RTX_Reserve_Sem( semaphoreId, &timeout ) == 0 )  // Wait for semaphore endlessly
    {
      printf( "\r\nTask %i: %s", RTX_Get_TaskID(), message );
      RTX_Release_Sem( semaphoreId );  // Release semaphore
    }
  }  // while( task1Running )

  // The task deletes itself after task1Running has been set to 0
  if( RTX_Delete_Task( RTX_Get_TaskID() ) != 0 )
  // Of course this is superfluous for this code will not be reached if the task
  // has been deleted...
  {
    printf( "\r\nError: Task %i could not be deleted.", RTX_Get_TaskID() );
  }
}



/******************************************************************************
* main()
*
* Creates the two tasks, the semaphore and the event group.
* Triggers the event each time the user presses Return.
* Deletes tasks, semaphore and event group when the user presses Esc.
******************************************************************************/
#pragma argsused
int main(
          int argc,
          char *argv[] )
{
  unsigned char key = 0;  // Key pressed by user


  // Set STDIO focus to USER/ APPLICATION
  BIOS_Set_Focus( FOCUS_APPLICATION );

  // Say hello
  printf( "\r\nEvent - IPC@Chip RTOS example" );


  // Create the semaphore
  if( RTX_Create_Sem( &semaphoreId, semaphoreName, -1 /*Resource semaphore*/ ) != 0 )
  {
    printf( "\r\nError: Could not create semaphore."
            "\r\n" );
    return 1;
  }

  // Create the event group
  if( RTX_Create_EventGroup( &eventgroupId, eventgroupName, 0 ) != 0 )
  {
    printf( "\r\nError: Could not create event group."
            "\r\n" );
    return 1;
  }


  // Create tasks
  if( RTX_Create_Task( &task0Id, &task0DefBlock ) != 0 )
  {
    printf( "\r\nError: Could not create task 0."
            "\r\n" );
    return 1;
  }
  if( RTX_Create_Task( &task1Id, &task1DefBlock ) != 0 )
  {
    printf( "\r\nError: Could not create task 1."
            "\r\n" );
    return 1;
  }


  // Main loop
  printf( "\r\nPress Return to trigger the event, ESC exits program"
          "\r\n" );
  while( key != 27 )
  {
    key = getch();
    if( key == 13 )
    {
      // Trigger event
      if( RTX_Signal_Events( eventgroupId, 0x0001, 0x0001 ) != 0 ||
          RTX_Signal_Events( eventgroupId, 0x0001, 0x0000 ) != 0 )
        printf( "\r\nError: Could not trigger event." );
    }
  }


  // Wake up tasks to let them delete themselves
  task0Running =
  task1Running = 0;

  if( RTX_Signal_Events( eventgroupId, 0x0001, 0x0001 ) != 0 ||
      RTX_Signal_Events( eventgroupId, 0x0001, 0x0000 ) != 0 )
    printf( "\r\nError: Could not delete tasks." );

  RTX_Sleep_Time( 2000 );  // Give them some time


  // Delete the semaphore
  if( RTX_Delete_Sem( semaphoreId ) != 0 )
    printf( "\r\nError: Could not delete semaphore." );

  if( RTX_Delete_EventGroup( eventgroupId ) != 0 )
    printf( "\r\nError: Could not delete event group." );


  // Say bye
  printf( "\r\nGood bye!"
          "\r\n" );

  // Set STDIO focus to BOTH
  BIOS_Set_Focus( FOCUS_BOTH );

  return 0;
}



// End of file
