/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:    dma_cnt.c
* Function:  This example shows the usage of the direct memory access (DMA) with
*            the SC13 on the DK50 Evaluation Board. To use this software
*            example you need some litle hardware, shown in the dma example
*            schematic.
*            The program uses TMROUT0 to trigger an external binary counter. The
*            binary output of the counter can be read from i/o address 0x640.
*            TMROUT1 is used to trigger the DMA channel 1 at the DRQ1 pin.
*            Through the program you can sample with a constant frequency data
*            at port 0x640. The frequency of the counter is independent of the
*            sampling rate.
*            T(on) is the time of holding DRQ1 at high. If T(on)
*            is longer than one read access to the port, more than 1 Byte is
*            read at this sampling point. If T(on) is shorter than on read access,
*            only 1 Byte is read.
*            You can vary the read access time by the number of wait states for
*            the access.
*
*            example parameters:
*            A:\>dma_cnt 500000 25000 1 1
*            generates a count frequency of 416666 Hz and samples each twentieth output
*
*            A:\>dma_cnt 416666 41666 1 1
*            generates a count frequency of 416666 Hz and samples each tenth output
*
* Note:      Not each TMROUT frequency is possible. The clock divisor must be
*            by an integer.
*
* Author:    Marco Nesseldreher
*
*******************************************************************************
*
* $Header: dma_cnt.c, 2, 26.04.2004 14:50:07, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdlib.h>
#include <conio.h>
#include <stdio.h>
#include "clib.h"

/******************************************************************************
* Defines
******************************************************************************/
#define DMA_COUNT 20                // number of sampled values
#define RESERVED  0x0010

/******************************************************************************
* Variables
******************************************************************************/
int dma_finished = 0;

/******************************************************************************
* Interrupt handler
******************************************************************************/
void huge dma_int_handler (void)          // DMA interrupt service routine
{
   outportb(0x600, inportb(0x600)+1);
   dma_finished = 1;
}

/***************************************************************************/
// Display command line options
/***************************************************************************/
void usage (void)
{
   unsigned long timer_freq;

   timer_freq = hal_get_frequencies(GET_FRQ_TIMER);

   printf("\nProgram initializes DMA1 and starts TMROUT 1 and 2");
   printf("\nUsage: dma_cnt <counter freq> <sample freq> <T(on)> <wait states>");
   printf("\n counter freq: [Hz]");
   printf("\n sample freq : [Hz]");
   printf("\n T(on) time: [n * %lu ns]", 500000000L/(timer_freq));
   printf("\n number of wait states : 0,1,2,3,5,7,9,15,24,41,...");
}

/******************************************************************************
* Main
******************************************************************************/
void main(int argc, char *argv[])
{
   int i, wait_states, ws;
   unsigned long timer_freq, maxcount_long, counter_freq, sample_freq;
   unsigned char buffer[DMA_COUNT+10];
   unsigned Ton, Toff, maxcount;
   Dma_Info dma_info;

   if (argc >= 5)
   {
      counter_freq = atol(argv[1]);
      sample_freq  = atol(argv[2]);
      Ton          = atoi(argv[3]);
      ws           = atoi(argv[4]);

      if (Ton <= 0)
      {
         printf("\nT(on) > 0 !\n");
         usage();
         return;
      }

      switch (ws)
      {
         case 0:   wait_states = PFE_0_WAIT;   break;
         case 1:   wait_states = PFE_1_WAIT;   break;
         case 2:   wait_states = PFE_2_WAIT;   break;
         case 3:   wait_states = PFE_3_WAIT;   break;
         case 5:   wait_states = PFE_5_WAIT;   break;
         case 7:   wait_states = PFE_7_WAIT;   break;
         case 9:   wait_states = PFE_9_WAIT;   break;
         case 15:  wait_states = PFE_15_WAIT;  break;
         case 24:  wait_states = PFE_24_WAIT;  break;
         case 41:  wait_states = PFE_41_WAIT;  break;
         case 74:  wait_states = PFE_74_WAIT;  break;
         case 123: wait_states = PFE_123_WAIT; break;
         case 157: wait_states = PFE_157_WAIT; break;
         case 175: wait_states = PFE_175_WAIT; break;
         case 201: wait_states = PFE_201_WAIT; break;
         case 255: wait_states = PFE_255_WAIT; break;
         default:
         {
            printf("\nThis number of wait states is not possible");
            return;
         }
      }
   }
   else
   {
     usage();
     return;
   }

   // Get timer base freq.
   timer_freq = hal_get_frequencies(GET_FRQ_TIMER);

   Toff = (unsigned)(timer_freq/(sample_freq)*2) - Ton;
   if ((Toff>32768L) || (Toff == 0))
   {
     printf("\nDMA/sampling frequency not possible\n");
     usage();
     goto end;
   }
   maxcount_long = ((unsigned long)timer_freq/counter_freq);
   if ((maxcount_long>65535L) || (maxcount_long == 0))
   {
     printf("\nExternal counter frequency not possible\n");
     usage();
     goto end;
   }
   maxcount = (unsigned)maxcount_long;
   printf("\nTMROUT0 frequency to trigger external counter:%7lu Hz = divider:%5u",
          timer_freq/maxcount, ((unsigned long)timer_freq/counter_freq));
   printf("\nTMROUT1 frequency to trigger DMA/sampling    :%7lu Hz = divider:%5u",
          ((unsigned long)timer_freq/(Ton + Toff)*2), ((unsigned long)timer_freq/(sample_freq)*2));
   printf("\nDuty cycle : T(on)=%d  T(off)=%u", Ton, Toff);
   printf("\nWait states: %d\n", ws);

   // set focus to application
   BIOS_Set_Focus ( FOCUS_APPLICATION );

   pfe_enable_bus (0xffff, 1);
   pfe_enable_pcs ( 6 );
   pfe_set_wait_states2 ( wait_states );        // set wait states of PCS5..6
   pfe_enable_pio ( 1, 5 );                     // counter Reset (MR) auf low

   if (pfe_enable_external_dma (1))
   {
      printf("\nDMA enable failed");
      printf("\nensure that DMA is disabled for COM port in chip.ini");
      goto end;
   }

   hal_install_isr(6, 1, dma_int_handler);

   // TMROUT1 for triggering DRQ
   pfe_enable_timer(0x0c);                      // enable TMROUT1
   hal_init_timer(1, 1, Ton);                   // continous
   hal_set_duty_cycle_waveform ( 1, 1, Toff );  // length of DRQ impulse pause

   // TMROUT0 for triggering external counter
   pfe_enable_timer(0x03);                         // enable TMROUT0
   hal_init_timer(0, 1, maxcount);                 // continous
   hal_set_duty_cycle_waveform ( 0, 1, maxcount);  // symetric cycling
   hal_start_timer(0);                             // start TMROUT0

   // DMA Init values
   dma_info.count    = DMA_COUNT;
   dma_info.dest.mem = buffer;
   dma_info.src.io   = 0x640;
   dma_info.ctrl_reg = DMA_PRIO + DMA_SYNCDEST + DMA_DOINTR + RESERVED +
                       DMA_DESTINC + DMA_DESTMEM;

   printf("\nStart trigger for external counter");
   hal_start_timer(1);                          // start TMROUT1

   dma_finished = 0;
   if (hal_start_dma ( 1, &dma_info ))
   {
      printf("\nStarting DMA failed");
      goto end;
   }
   else printf("\nStart DMA");

   while(!dma_finished)                        // wait until DMA count is reached
   {
      RTX_Sleep_Time(10);
      if (kbhit())
      {
         printf("\nExit DMA transfer");
         hal_stop_dma(1);
         goto end;
      }
   }
   hal_get_dma_info (1, &dma_info);

   for (i = 0; i < DMA_COUNT; i++)
   {
     printf("\nsampled data: %03d", buffer[i]);
   }
   printf("\nDMA count: 0x%02x", dma_info.count);

end:
   hal_install_isr(6, 0, NULL);           // deinstall DMA ISR
   BIOS_Set_Focus ( FOCUS_BOTH );         // set focus to both
}
