/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
* Module:     myipd.c
* Function:   The My-IP-Daemon combines the HTTP-example with the FTP-example.
*             It determines the IP-address of the chip that is valid on the
*             internet. This address can be different from what Get_IPConfig()
*             delivers e.g. if the chip is connected to the internet through a
*             router. To do so a CGI-script is called from the webserver of
*             Beck IPC that simply shows the IP-address of the caller. The CGI-
*             script which is located at beck2.beck-ipc.com/myip/myip.cgi looks
*             like this:
*               #!/bin/sh
*               echo Content-type: text/plain
*               echo
*               echo MyIP:$REMOTE_ADDR
*             Once the IP-address is extracted from the script's output a small
*             HTML-file is generated which shows the address. This file is then
*             send to a FTP-server.
*             To always keep this HTML-file up-to-date, myipd periodically
*             checks if the IP-address has changed and eventually updates the
*             HTML-file again.
* Notes:      To make myipd run correctly you'll have to add a new section to
*             your chip.ini. The section must be named MYIPD and contain the
*             following strings:
*               DNS_SERVER_IP  The IP address of a DNS-Server that can be used
*                              to resolve the names of the server with the
*                              CGI-script and of the FTP-server
*               HTTP_URL       The URL of the CGI-script that prints the IP-
*                              address
*               FTP_HOST       The hostname of the FTP-server
*               FTP_FILENAME   The filename that the HTML-file showing the IP-
*                              address will have on the FTP-server
*               FTP_USER,
*               FTP_PASSWORD   Login information for the FTP-server
*
*******************************************************************************
*
* $Header: myipd.c, 1, 04.05.2004 15:50:13, Jan Schatz$
*
******************************************************************************/



#include <stdio.h>
#include <string.h>
#include <clib.h>
#include "httpcli.h"
#include "ftplib.h"
#include "dns.h"



/******************************************************************************
* Constants
******************************************************************************/
#define DEBUG                                  // Debug switch
#define BUFFERSIZE       1024
#define INTERVAL         (unsigned int)60000L  // Interval after which the IP-
                                               // address will be checked again
#define HTML_TEMPLATE    "<html>\r\n" \
                         "<head>\r\n" \
                         "  <title>IPC@Chip</title>\r\n" \
                         "</head>\r\n" \
                         "<body>\r\n" \
                         "  <h1>IPC@Chip</h1>\r\n" \
                         "  <p>My IP-address is %s.</p>\r\n" \
                         "</body>\r\n" \
                         "</html>\r\n"         // Template for the HTML-file



/******************************************************************************
* Global variables
******************************************************************************/
char buffer[BUFFERSIZE];



/******************************************************************************
* main
******************************************************************************/
#pragma argsused
int main( int argc, char *argv[] )
{
  int bufferSize = 0;                   // Size of data in buffer
  static char ftpIp[16] = "",           // IP-address of FTP-server
              myIpOld[16] = "0.0.0.0",  // My old IP-address
              dnsServerIp[16],          // IP-address of DNS-server
              httpUrl[128],             // URL to get my IP-address from
              ftpHost[128],             // Hostname of FTP-server
              ftpFilename[128],         // Filename of HTML-file on FTP-server
              ftpUser[32],
              ftpPassword[32];          // Login information for FTP-server
  char *myIp = myIpOld,                 // My IP-address on the internet
       *tmpFilename;                    // Name of temporary file
  FILE *tmpFile;                        // File-handle for temporary file
  int socket,                           // Socket for FTP-control-connection
      error;                            // Result of TCP/IP-API-calls
  unsigned long dummy;                  // Dummy variable


  // Get parameters from chip.ini
  if( BIOS_Get_Ini_String( "MYIPD", "DNS_SERVER_IP", dnsServerIp,
                           sizeof( dnsServerIp ) - 1 ) == 0 ||
      BIOS_Get_Ini_String( "MYIPD", "HTTP_URL", httpUrl,
                           sizeof( httpUrl ) - 1 ) == 0 ||
      BIOS_Get_Ini_String( "MYIPD", "FTP_HOST", ftpHost,
                           sizeof( ftpHost ) - 1 ) == 0 ||
      BIOS_Get_Ini_String( "MYIPD", "FTP_FILENAME", ftpFilename,
                           sizeof( ftpFilename ) - 1 ) == 0 ||
      BIOS_Get_Ini_String( "MYIPD", "FTP_USER", ftpUser,
                           sizeof( ftpUser ) - 1 ) == 0 ||
      BIOS_Get_Ini_String( "MYIPD", "FTP_PASSWORD", ftpPassword,
                           sizeof( ftpPassword ) - 1 ) == 0 )
  {
    printf( "\r\nmyipd error: Configuration incomplete."
            "\r\nYou must define the following in your chip.ini:"
            "\r\n[MYIPD]"
            "\r\nDNS_SERVER_IP=<DNS-server>"
            "\r\nHTTP_URL=<URL to get chip's IP-address from>"
            "\r\nFTP_HOST=<Hostname of FTP-server>"
            "\r\nFTP_FILENAME=<Name of HTML-file on FTP-server>"
            "\r\nFTP_USER=<Username for FTP-login>"
            "\r\nFTP_PASSWORD=<Password for FTP-login>" );
    return 1;
  }


  // Main loop
  // Check if my IP-address has changed and eventually update HTML-file on
  // FTP-server
  for( ; TRUE; api_sleep( INTERVAL ) )
  {
    #ifdef DEBUG
    printf( "\r\nmyipd: Checking for new IP-address..." );
    #endif

    // Save my IP-address
    strncpy( myIpOld, myIp, 15 );

    // Get my IP-address from URL
    if( ( bufferSize = httpGet( httpUrl, buffer, BUFFERSIZE - 1, dnsServerIp ) ) <= 0 )
    {
      printf( "\r\nmyipd error: Couldn't get %s.", httpUrl );
      continue;
    }
    buffer[bufferSize] = '\0';  // Make make string of buffer[]

    // Extract IP-address
    myIp = strstr( buffer, "MyIP:" );
    myIp += 5;  // Skip text "MyIP:"
    ( *strpbrk( myIp, "\r\n " ) ) = '\0';  // Cut everything behind IP-address

    if( strlen( myIp ) > 15 )  // Check length of IP-address
    {
      printf( "\r\nmyipd error: Couldn't get my IP-address." );
      continue;
    }

    #ifdef DEBUG
    printf( "\r\nmyipd: Old IP-address: %s, new IP-address: %s", myIpOld, myIp );
    #endif


    // Check if HTML-file must be updated on FTP-server
    if( strcmp( myIp, myIpOld ) != 0 )
    {
      #ifdef DEBUG
      printf( "\r\nmyipd: Updating..." );
      #endif

      // Create temporary file
      tmpFilename = tmpnam( NULL );
      if( ( tmpFile = fopen( tmpFilename, "w" ) ) == NULL )
      {
        printf( "\r\nmyipd error: Could not open temporary file." );
        continue;
      }

      if( fprintf( tmpFile, HTML_TEMPLATE, myIp ) == EOF )
      {
        fclose( tmpFile );
        remove( tmpFilename );

        printf( "\r\nmyipd error: Could not write to temporary file." );
        continue;
      }
      fclose( tmpFile );


      // Send temporary file via FTP
      if( gethostbyname( dnsServerIp, ftpHost, &dummy, NULL, ftpIp ) != 1 )
      {
        remove( tmpFilename );

        printf( "\r\nmyipd error: Could not resolve %s.", ftpHost );
        continue;
      }

      if( ( socket = ftpOpenControlConnection( ftpIp ) ) == -1 )
      {
        remove( tmpFilename );

        printf( "\r\nmyipd error: Could not connect to %s.", ftpIp );
        continue;
      }

      if( !ftpLogin( socket, ftpUser, ftpPassword, NULL ) )
      {
        closesocket( socket, &error );
        remove( tmpFilename );

        printf( "\r\nmyipd error: Could not login to %s", ftpIp );
        continue;
      }

      if( !ftpSendFile( socket, ftpFilename, tmpFilename, FTP_TYPE_ASCII_NONPRINT, TRUE ) )
      {
        closesocket( socket, &error );
        remove( tmpFilename );

        printf( "\r\nmyipd error: Could not send temporary file to %s", ftpIp );
        continue;
      }

      ftpLogout( socket );


      // Remove temporary file
      if( remove( tmpFilename ) != 0 )
        printf( "\r\nWarning: Could not remove temporary file." );

      #ifdef DEBUG
      printf( "\r\nmyipd: Done" );
      #endif
    }  // Update of HTML-file
  }  // Main loop
}



// End of file
