/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     pwd.c
* Function:   Example to show the functionality of the PWD option
*
*             Furher description:
*             The PWD function of the IPC@CHIP allows to measure the width
*             (time of high/low level) of a signal on the PWD Pin.
*             Atfer enabling the function by using the hardware API,
*             int 2, int 4, timer 0 and timer 1 will be controlled internal
*             by the CPU.
*             While the signal is low, the timer 1 is active and counts with
*             the PWD timer frequency. The frequence will be returned by
*             calling the function 'Get Frequency' in the hardware API.
*             On a rising edge of the signal on the PWD pin, int 2 will be
*             called. The interrupt routine reads the counter of timer 1,
*             save it to a global variable and resets the timer.
*             Now (while the high level of the signal) timer 0 is active.
*             At the falling edge int 4 will be called. The interrupt
*             routine reads out the counted events of timer 0, save it to a
*             global variable and resets timer 0.
*
*             Depending on the PWD timer frequency we can generate the
*             time of the low level and the high level.
*
*             ms = [Counted Events] * [PWD Timer Frequence] / 1000
*
*******************************************************************************
*
* $Header: pwd.c, 2, 31.03.2004 11:15:55, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include "clib.h"

/******************************************************************************
* Variables
******************************************************************************/
volatile unsigned tmr0cnt                = 0;
volatile unsigned tmr1cnt                = 0;
volatile unsigned long width_low_level   = 0;   // time of low level  (in Hz)
volatile unsigned long width_high_level  = 0;   // time of high levl (in Hz)


/******************************************************************************
* Prototypes
******************************************************************************/
void huge int2_handler(void);
void huge int4_handler(void);
void huge tmr0_handler(void);
void huge tmr1_handler(void);

/******************************************************************************
* Interrupt handlers
******************************************************************************/
void huge tmr0_handler(void)
{
  tmr0cnt++;
}

void huge tmr1_handler(void)
{
  tmr1cnt++;
}

void huge int2_handler(void)
{
  unsigned counter;

  // Read timer1 count
  counter = hal_read_timer_count (1, NULL);

  // save counted timers ( edge width in Hz )
  width_low_level  = tmr1cnt;
  width_low_level *= 0x10000L;
  width_low_level += counter;

  // Reset timer1 count
  hal_write_timer_count (1, 0);
  tmr1cnt = 0;
}

void huge int4_handler(void)
{
  unsigned counter;

  // Read timer0 count
  counter = hal_read_timer_count (0, NULL);

  // save counted timers ( edge width in Hz )
  width_high_level  = tmr0cnt;
  width_high_level *= 0x10000L;
  width_high_level += counter;

  // Reset timer0 count
  hal_write_timer_count (0, 0);
  tmr0cnt = 0;
}


/******************************************************************************
* Main
******************************************************************************/
void main(void)
{
   int i=0;                                             // counter
   float ms_level_width;                                // level width in ms
   unsigned long PWD_timer_frq;                         // PWD timer frequence

   // Set int2 handler
   hal_install_isr (2, 1, int2_handler);

   // Set int4 handler
   hal_install_isr (4, 1, int4_handler);

   // Set timer0 handler
   hal_install_isr (8, 1, tmr0_handler);

   // Set timer1 handler
   hal_install_isr (9, 1, tmr1_handler);

   // Init timer0
   // continous, disable interrupt, int clock (011)
   // MaxCount count from 0000h-ffffh
   hal_init_timer(0, 0x03, 0);

   // Init timer1
   // continous, disable interrupt, int clock (011)
   // MaxCount count from 0000h-ffffh
   hal_init_timer(1, 0x03, 0);

   // Get pwd frequency
   PWD_timer_frq = hal_get_frequencies( GET_FRQ_PWD );

   // Enable PWD
   pfe_enable_pwd();

   // Start timer0
   hal_start_timer(0);

   // Start timer1
   hal_start_timer(1);

   for (i=0; i<60; i++)
   {
     // wait 1 sec
     RTX_Sleep_Time(1000);

     ms_level_width = ( (float)width_low_level  / (PWD_timer_frq/1000) );
     printf("\r\nLow  Level = %04.4f ms", ms_level_width );

     ms_level_width = ( (float)width_high_level / (PWD_timer_frq/1000) );
     printf("\r\nHigh Level = %04.4f ms", ms_level_width );
   }

   // Deinstall int2 handler
   hal_install_isr (2, 0, NULL);

   // Deinstall int4 handler
   hal_install_isr (4, 0, NULL);

   // stop timer0
   hal_stop_timer(0);

   // stop timer1
   hal_stop_timer(1);

   // Deinstall timer0 handler
   hal_install_isr (8, 0, NULL);

   // Deinstall timer1 handler
   hal_install_isr (9, 0, NULL);

   // deinit PWD (set to PIO)
   pfe_enable_pio(6, PIO_I);

   printf("\nDone.\n");
}

